/**
  \file  properties.c
*/
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include "properties.h"

/**
 \todo
 This version is far from perfect: 
 Option -a expects an argument for the author's name.
 It complains only if the name is missing if -a is the
 last option supplied. E.g.:
  -a -i -t
 will happily add -i as the name of the author.
*/
t_properties *getProperties(int argc, char *argv[])
{
  int           i;
  
  t_properties *myProps = readRcFile();
  if (myProps == NULL)
     myProps = getDefaultProperties();
  if (argc != 1) 
     {
        printf("Looking for options\n ");
        i = 1;
        while (i < argc) 
          {
            if (strcmp(argv[i], "-t") == 0)
               {
                 myProps->pr_time = 1;
               }
              else
            if (strcmp(argv[i], "-cv") == 0)
               {
                 myProps->pr_comments = COMMENTS_VERBOSE;
               }
              else
            if (strcmp(argv[i], "-cs") == 0)
               {
                 myProps->pr_comments = COMMENTS_SHORT;
               }
              else
            if (strcmp(argv[i], "-cn") == 0)
               {
                 myProps->pr_comments = COMMENTS_NONE;
               }
              else
            if (strcmp(argv[i], "-a") == 0)
               {
                i++;
                if (i >= argc)
                   {
                    fprintf(stderr, "ERROR: -a need author!\n");
                    exit(1);
                   }
                strcpy(myProps->pr_author, argv[i]);
               }
              else
            if (strcmp(argv[i], "-gv") == 0)
               {
                 myProps->pr_version = 1;
               }
              else
            if (strcmp(argv[i], "-gnv") == 0)
               {
                 myProps->pr_version = 0;
               }
            if (strcmp(argv[i], "-i") == 0)
               { 
                 i++;
                 if (i >= argc)
                    { 
                      fprintf(stderr, "ERROR: No include file specified\n");
                      exit(1);
                    }
                  else strcpy(myProps->pr_include, argv[i]);
               }
              else fprintf(stderr, "WARNING: Unknown option %s\n", argv[i]);
            i++;
          }
     }
  return myProps;
}

t_properties *readRcFile(void)
{
  return NULL;
}

t_properties *getDefaultProperties(void)
{
  t_properties *defaults = (t_properties *) malloc(sizeof(t_properties));
  
  if (defaults == NULL)
     {
       fprintf(stderr,"Error: Out of Memory\n");
       exit(1);
     }
  // 1. Author is unknown
  defaults->pr_author[0] = (char) 0; 
  // 2. enter origin's versions
  defaults->pr_version = 1;
  // 3. enter time stamp of creation
  defaults->pr_time = 0;
  // 4. Default license.txt as license source
  strcpy(defaults->pr_licensefile, "license.txt");
  // 5. No additional comments
  defaults->pr_list = NULL;
  // 6. Short comments
  defaults->pr_comments = COMMENTS_SHORT;
  // 7. Include file
  defaults->pr_include[0] = (char) 0;
  return defaults;
}

void get_version_line(char *version)
{
  char     *content, *ptr;
  struct stat entry;
  int     size, filed;
  
  if (stat("CVS/Entries", &entry) == -1)
     {
       fprintf(stderr, "Warning: Please run generate from the CVS main dir");
       strcpy(version, "unknown");
       return;
     }
  content = (char *) malloc(entry.st_size);
  filed= open("CVS/Entries", O_RDONLY);
  if (filed == -1)
     {
       fprintf(stderr, "Warning: Can't open CVS/Entries");
       strcpy(version, "unknown");
       return;
     }
  read(filed, content, entry.st_size);
  // Find the beginning of objects.c
  ptr = strstr(content, "/objects.c/");
  if (ptr == NULL)
    {
       fprintf(stderr, "Warning: Can't find objects.c entry");
       strcpy(version, "unknown");
       close(filed);
       return;
    }
  ptr = ptr + strlen("/objects.c/");
  strncpy(version, ptr, 20);
  ptr = strchr(version, '/');
  *ptr = (char) 0;
  close(filed);
}
