/*
	nouveau_reg.h generator
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/sendfile.h>
#include <unistd.h>

#include "rexml.h"

/* which column should be put the value ? */
#define INDENT_VALUE_COL	88
#define TAB_SIZE 8

void print_header(void);
void print_footer(void);
void print_indented(const char *line);
void print_objects(void);
void print_object_reg32(xmlNodePtr node, xmlChar *parent_name);
void print_bitfield(xmlNodePtr node, xmlChar *parent_name, xmlChar *offset_name);
void print_enums(xmlChar *enum_name, const char *hdr, int shift);

/* tmp buffer */
char tmp[512];

int main(int argc, char **argv)
{
	const char *filename_dbxml = "renouveau.xml";
	int i;

	for(i=0; i<argc; i++) {
		if (strcmp("-dbxml", argv[i])==0) {
			filename_dbxml = argv[i+1];
		}
		if (strcmp("-help", argv[i])==0) {
			printf("Parameters:\n");
			printf(" -dbxml <filename.xml> (xml database for parser)\n");
			exit(0);
		}
	}

	if (rexmlInit(filename_dbxml)<0) {
		fprintf(stderr, "Can not load xml database %s\n", filename_dbxml);
		return 1;
	}

	print_header();
	print_objects();
	print_footer();

	rexmlShutdown();
	return 0;
}

void print_header(void)
{
	int filed=open("license.txt",O_RDONLY);
	char c;
	while(read(filed,&c,1)>0)
		write(1,&c,1);
	close(filed);
	printf("\n#ifndef NOUVEAU_REG_H\n");
	printf("#define NOUVEAU_REG_H 1\n\n\n");
}

void print_footer(void)
{
	printf("#endif /* NOUVEAU_REG_H */\n");
}

void print_indented(const char *line)
{
	int i = strlen(line);

	printf("%s", line);
	while (i<INDENT_VALUE_COL) {
		printf("\t");
		i+=TAB_SIZE;
	}
}

/* Dump all objects nodes */

void print_objects(void)
{
	xmlXPathObjectPtr pathobj;
	xmlNodePtr arrsz;

	pathobj = rexmlSearchNode(BAD_CAST "/renouveau");
	if (!pathobj) {
		return;
	}

	arrsz = pathobj->nodesetval->nodeTab[0]->children;

	for(; arrsz; arrsz=arrsz->next) {
		xmlChar *attId, *attName;
		int id;

		if ( (arrsz->type != XML_ELEMENT_NODE) || !arrsz->name) {
			continue;
		}

		if (xmlStrcmp(arrsz->name, BAD_CAST "object")!=0) {
			continue;
		}

		attId = xmlGetProp(arrsz, BAD_CAST "id");
		attName = xmlGetProp(arrsz, BAD_CAST "name");

		if (attId && attName) {
			if (sscanf((char*)attId, "0x%x", &id)) {
				sprintf(tmp, "#define %s", attName);
				print_indented(tmp);
				printf("0x%08x\n\n", id);
				print_object_reg32(arrsz, attName);
				printf("\n\n");
			}
		}

		if (attName) {
			xmlFree(attName);
		}
		if (attId) {
			xmlFree(attId);
		}
	}

	xmlXPathFreeObject(pathobj);
}

/* Dump all reg32 nodes of an object */

void print_object_reg32(xmlNodePtr node, xmlChar *parent_name)
{
	xmlNodePtr arrsz = node->children;

	for(; arrsz; arrsz=arrsz->next) {
		xmlChar *attOffset, *attName, *attSize, *attStride, *attType;

		if ( (arrsz->type != XML_ELEMENT_NODE) || !arrsz->name) {
			continue;
		}

		if (xmlStrcmp(arrsz->name, BAD_CAST "reg32")!=0) {
			continue;
		}

		attOffset = xmlGetProp(arrsz, BAD_CAST "offset");
		attName = xmlGetProp(arrsz, BAD_CAST "name");
		attSize = xmlGetProp(arrsz, BAD_CAST "size");
		attStride = xmlGetProp(arrsz, BAD_CAST "stride");
		attType = xmlGetProp(arrsz, BAD_CAST "type");

		if (attOffset && attName) {
			int offset, size = 1, stride = 4;
			if (attSize) {
				if (!sscanf((char*)attSize, "%d", &size)) {
					size = 1;
				}
			}
			if (attStride) {
				if (!sscanf((char*)attStride, "%d", &stride)) {
					stride = 4;
				}
			}
			if (sscanf((char*)attOffset, "0x%x", &offset)) {
				if (size>1) {
					/* Array */
					sprintf(tmp, "#define  %s_%s(x)", parent_name, attName);
					print_indented(tmp);
					printf("(0x%08x+((x)*%d))\n", offset, stride);

					sprintf(tmp, "#define  %s_%s__SIZE", parent_name, attName);
					print_indented(tmp);
					printf("0x%08x\n", size);
				} else {
					sprintf(tmp, "#define  %s_%s", parent_name, attName);
					print_indented(tmp);
					printf("0x%08x\n", offset);
				}
				if (attType) {
					if (xmlStrcmp(attType, BAD_CAST "bitfield")==0) {
						print_bitfield(arrsz, parent_name, attName);
					} else
					if (xmlStrcmp(attType, BAD_CAST "enum") == 0) {
						xmlChar *enumName = xmlGetProp(arrsz, BAD_CAST "enum_name");
						if (enumName) {
							sprintf(tmp, "#define   %s_%s", parent_name, attName);
							print_enums(enumName, tmp, 0);
							xmlFree(enumName);
						}
					}
				}
			}
		}

		if (attType) {
			xmlFree(attType);
		}
		if (attStride) {
			xmlFree(attStride);
		}
		if (attSize) {
			xmlFree(attSize);
		}
		if (attName) {
			xmlFree(attName);
		}
		if (attOffset) {
			xmlFree(attOffset);
		}
	}
}

void print_bitfield(xmlNodePtr node, xmlChar *parent_name, xmlChar *offset_name)
{
	xmlNodePtr arrsz = node->children;

	for(; arrsz; arrsz=arrsz->next) {
		xmlChar *attValue, *attName, *attType;
		int highbit = -1, lowbit = -1;
		unsigned int mask;

		if ( (arrsz->type != XML_ELEMENT_NODE) || !arrsz->name) {
			continue;
		}

		if (xmlStrcmp(arrsz->name, BAD_CAST "bitfield")!=0) {
			continue;
		}

		attValue = xmlGetProp(arrsz, BAD_CAST "high");
		if (attValue) {
			highbit = atoi((char*)attValue);
			xmlFree(attValue);
		}

		attValue = xmlGetProp(arrsz, BAD_CAST "low");
		if (attValue) {
			lowbit = atoi((char*)attValue);
			xmlFree(attValue);
		}

		if ((lowbit<0) || (highbit<0)) {
			continue;
		}

		attName = xmlGetProp(arrsz, BAD_CAST "name");
		if (!attName) {
			continue;
		}

		if (lowbit == highbit) {
			sprintf(tmp, "#define   %s_%s_%s", parent_name, offset_name, attName);
			print_indented(tmp);
			printf("(1 << %2d)\n", lowbit);
		} else {
			sprintf(tmp, "#define   %s_%s_%s_SHIFT", parent_name, offset_name, attName);
			print_indented(tmp);
			printf("%d\n", lowbit);

			mask = (0xffffffffUL >> (31-highbit)) & (0xffffffffUL << lowbit);

			sprintf(tmp, "#define   %s_%s_%s_MASK", parent_name, offset_name, attName);
			print_indented(tmp);
			printf("0x%08x\n", mask);
		}

		attType = xmlGetProp(arrsz, BAD_CAST "type");
		if (attType && xmlStrcmp(attType, BAD_CAST "enum") == 0) {
			xmlChar *enumName = xmlGetProp(arrsz, BAD_CAST "enum_name");
			if (enumName) {
				sprintf(tmp, "#define    %s_%s_%s", parent_name, offset_name, attName);
				print_enums(enumName, tmp, lowbit);
				xmlFree(enumName);
			}
		}

		if (attType)
			xmlFree(attType);

		xmlFree(attName);
	}
}

void print_enums(xmlChar *enumName, const char *hdr, int shift)
{
	xmlXPathObjectPtr pathobj;
	xmlNodePtr arrsz;

	pathobj = rexmlSearchNode(BAD_CAST "/renouveau/enums");
	if (!pathobj)
		return;

	arrsz = pathobj->nodesetval->nodeTab[0]->children;
	if (!arrsz)
		return;

	for(; arrsz; arrsz=arrsz->next) {
		xmlNodePtr list;
		xmlChar *listName;

		if (xmlStrcmp(arrsz->name, BAD_CAST "list") != 0)
			continue;

		listName = xmlGetProp(arrsz, BAD_CAST "name");
		if (!listName)
			continue;

		if (xmlStrcmp(listName, enumName) != 0) {
			xmlFree(listName);
			continue;
		}

		for (list = arrsz->children; list; list = list->next) {
			char edef[512];
			xmlChar *thisName, *thisValue;;

			thisName = xmlGetProp(list, BAD_CAST "name");
			thisValue = xmlGetProp(list, BAD_CAST "value");

			if (thisName && thisValue) {
				unsigned int value = strtol((char*)thisValue, NULL, 0) << shift;
				sprintf(edef, "%s_%s", hdr, thisName);
				print_indented(edef);
				printf("0x%08x\n", value);
			}

			if (thisName)
				xmlFree(thisName);
			if (thisValue)
				xmlFree(thisValue);
		}

		xmlFree(listName);
		break;
	}

	xmlXPathFreeObject(pathobj);
}
