/*
	renouveau.xml related functions
*/

#ifndef REXML_H
#define REXML_H 1

#include <libxml/xmlversion.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xmlreader.h>

/*--- Types ---*/

typedef struct {
	void (*printString)(const char *value);
	void (*printHeader)(int size, int channel, int offset, int increment, int value);
	void (*printParam)(int channel, int offset, int value);
} rexmlParseFuncs_t;

/*--- Functions ---*/

/*
	Initialise xml database from a file
		Returns 0 if success
*/
int rexmlInit(const char *filename);

/*
	Close xml database
*/
void rexmlShutdown(void);

/*
	Search a node given a path
		Returns NULL if failed
*/
xmlXPathObjectPtr rexmlSearchNode(xmlChar *xpath);

/*
	Parse a dump
		Calls needed functions for each part of a dump
*/

void rexmlParse(const char *filename, rexmlParseFuncs_t *rexmlParseFuncs);

/*
	Search a node bitfield in given 'node' with 'attr_name' name attribute
		Return 0 if error, or parsed value from bitfield inside value
*/

int rexmlGetBitfield(xmlNodePtr node, const char *attr_name, int value);

/*
	Search a child node with given node name and attribute
*/

xmlNodePtr rexmlSearchNodeAttrib(xmlNodePtr parent, const char *node_name,
	const char *attr_name, const char *attr_value);

/*
	Return function node of an object given index
*/

xmlNodePtr rexmlGetFunctionNode(xmlNodePtr parent, int index,
	int *is_array, int *start_offset, int *stride);

/*
	Return object node from object_id
*/

xmlNodePtr rexmlGetObjectNode(int object_id);

#endif /* REXML_H */
